import bpy
import numpy
from .setting import *

class Particle:
    """This class create a particle with the following characteristics:
    Parameters
    ----------
    name : string
    position : array (x,y,z)
    rotation : array (yz,zx,xy)
    scale : array (sx,sy,sz)
    collection : string : (It is the name of the collection to which the particle belongs)
    """

    def __init__(self, name="particle", position=None, rotation=None, scale=None) -> None:
        self._name = name

        try:
            self._position = (
                position
                if position != None
                else [bpy.data.objects[self._name].location[i] for i in range(3)]
            )
            self._rotation = (
                rotation
                if rotation != None
                else [bpy.data.objects[self._name].rotation_euler[i] for i in range(3)]
            )
            self._scale = (
                scale
                if scale != None
                else [bpy.data.objects[self._name].scale[i] for i in range(3)]
            )
        except:
            pass

    def get_name(self) -> str:
        return self._name

    def set_name(self, name: str) -> None:
        bpy.data.objects[self._name].name = name
        self._name = name
        return None

    def get_position(self) -> list:
        return [bpy.data.objects[self._name].location[i] for i in range(3)]

    def set_position(self, position: list) -> None:
        if self._checking_the_type_and_the_dimensionality(position):
            bpy.data.objects[self._name].location = position
            self._position = position
        return None

    def get_rotation(self) -> list:
        return [bpy.data.objects[self._name].rotation_euler[i] for i in range(3)]

    def set_rotation(self, rotation: list) -> None:
        if self._checking_the_type_and_the_dimensionality(rotation):
            bpy.data.objects[self._name].rotation_euler = rotation
            self._rotation = rotation
        return None

    def get_scale(self) -> list:
        return [bpy.data.objects[self._name].scale[i] for i in range(3)]

    def set_scale(self, scale: list) -> None:
        if self._checking_the_type_and_the_dimensionality(scale):
            bpy.data.objects[self._name].scale = scale
            self._scale = scale
        return None
    
    def set_keyframe(self, data_path="location", frame = setting.get_frame()) -> None:
        bpy.data.objects[self._name].keyframe_insert(data_path = data_path, frame = frame)

    def get_vertices(self) -> list:
        bpy.ops.object.select_all(action="DESELECT")
        bpy.data.objects[self._name].select_set(True)
        bpy.context.view_layer.objects.active = bpy.data.objects[self._name]
        return [v.co for v in bpy.context.active_object.data.vertices]
    

    def set_vertices(self, new_coordinate: dict) -> None:
        if bpy.context.active_object.mode == "EDIT":
            bpy.ops.object.editmode_toggle()
        bpy.ops.object.select_all(action="DESELECT")
        bpy.data.objects[self._name].select_set(True)
        for vertice_id in new_coordinate.keys(): 
            bpy.context.active_object.data.vertices[vertice_id].co = new_coordinate[vertice_id]
        return None
    
    def set_keyframe_vertices(self, vertice_id, frame = setting.get_frame()) -> None:
        bpy.data.objects[self._name].data.vertices[vertice_id].keyframe_insert("co", frame = frame)
        return None


    @staticmethod
    def _checking_the_type_and_the_dimensionality(greatness) -> bool:
        if (len(greatness) == 3) and (
            isinstance(greatness, list) or isinstance(greatness, np.ndarray)
        ):
            return True
        else:
            print("You must provide as input a list or a numpy array of length 3!")
            return False

